<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01//EN"
"http://www.w3.org/TR/html4/strict.dtd">
<html>
	<head>
		<title>Authorize Token</title>
	</head>
	<body>
		<h1>Authorize Token</h1>
		
        <form action="{% url piston.authentication.oauth_user_auth %}" method="POST">
            {{ form.as_table }}
        </form>

	</body>
</html>
